@@ -1,13 +1,20 @@
+require "cinch/exceptions"
+
 module Cinch
   # @api private
+  # @since 1.1.0
   module ModeParser
+    # @param [String] modes The mode string as sent by the server
+    # @param [Array<String>] params Parameters belonging to the modes
+    # @param [Hash{:add, :remove => Array<String>}] param_modes
+    #   A mapping describing which modes require parameters
     def self.parse_modes(modes, params, param_modes = {})
       if modes.size == 0
-        raise InvalidModeString, 'Empty mode string'
+        raise Exceptions::InvalidModeString, 'Empty mode string'
       end
 
       if modes[0] !~ /[+-]/
-        raise InvalidModeString, "Malformed modes string: %s" % modes
+        raise Exceptions::InvalidModeString, "Malformed modes string: %s" % modes
       end
 
       changes = []
@@ -18,7 +25,7 @@ module Cinch
       modes.each_char do |ch|
         if ch =~ /[+-]/
           if count == 0
-            raise InvalidModeString, 'Empty mode sequence: %s' % modes
+            raise Exceptions::InvalidModeString, 'Empty mode sequence: %s' % modes
           end
 
           direction = case ch
@@ -30,11 +37,11 @@ module Cinch
           count = 0
         else
           param = nil
-          if param_modes[direction].include?(ch)
+          if param_modes.has_key?(direction) && param_modes[direction].include?(ch)
             if params.size > 0
               param = params.shift
             else
-              raise InvalidModeString, 'Not enough parameters: %s' % ch.inspect
+              raise Exceptions::InvalidModeString, 'Not enough parameters: %s' % ch.inspect
             end
           end
           changes << [direction, ch, param]
@@ -43,11 +50,11 @@ module Cinch
       end
 
       if params.size > 0
-        raise InvalidModeString, 'Too many parameters: %s %s' % [modes, params].inspect
+        raise Exceptions::InvalidModeString, 'Too many parameters: %s %s' % [modes, params]
       end
 
       if count == 0
-        raise InvalidModeString, 'Empty mode sequence: %r' % modes
+        raise Exceptions::InvalidModeString, 'Empty mode sequence: %s' % modes
       end
 
       return changes
